#pragma once

/*
 * A structure to represent a single node of the tree.
 * This custom type also remembers how many objects of its type have been constructed.
 */
struct TreeNode {
    int data;
    TreeNode *left;
    TreeNode *right;

    /*
     * Construct a tree node with optional data, left, and right children.
     */
    TreeNode(int data = 0, TreeNode *left = nullptr, TreeNode *right = nullptr) {
        this->data = data;
        this->left = left;
        this->right = right;
        instances++;
    }

    /*
     * Returns true if this node is a leaf (no children).
     */
    bool isLeaf() const {
        return left == nullptr && right == nullptr;
    }

    /* The code below here tracks how many TreeNodes are created and deleted,
     * so that we can ensure all of them are correctly freed.
     */
    ~TreeNode()                    { instances--; }

    static int instances;
};
