/*
 * CS 106X, Nick Troccoli
 * This file implements various operations on trees.
 * See treefunctions.h for the declarations.
 */

#include "treefunctions.h"

using namespace std;

/*
 * Outputs all nodes of the tree with the given node as its root to the
 * output stream, in in-order traversal.
 */
void print(TreeNode *node, ostream& out) {
    if (node != nullptr) {
        print(node->left, out);
        out << node->data << endl;
        print(node->right, out);
    }
}

/*
 * Returns the number of nodes in the given tree.
 */
int size(TreeNode *node) {
    if (node == nullptr) {
        // base case: empty tree
        return 0;
    } else {
        // recursive case: non-nullptr node with possible children
        return 1 + size(node->left) + size(node->right);
    }
}

/*
 * Returns true if the given tree contains the given value.
 */
bool contains(TreeNode *node, int value) {
    if (node == nullptr) {
        return false;   // base case: not found here
    } else if (node->data == value) {
        return true;    // base case: found it!
    } else {
        return contains(node->left, value) || contains(node->right, value);
    }
}

/*
 * Returns the given tree's height.
 * The height of an empty tree is defined to be 0.
 * An unempty tree's height is the length of the longest path from the root
 * node to any other node in the tree.
 */
int height(TreeNode *node) {
    if (node == nullptr) {
        return 0;
    } else {
        return 1 + max(height(node->left), height(node->right));
    }
}

/*
 * Deletes all memory associated with the tree at the given root.
 */
void deleteTree(TreeNode *node) {
    if (node != nullptr) {
        deleteTree(node->left);
        deleteTree(node->right);
        delete node;
    }
}


